////////////////////////////////////////////////////
// Sensor-Driven Neural Controller.cpp consists of 
// -Initial Serial function
// -Initial All parameters
// -Joystick Control
// -GetSensor = reading sensor signals from A/D convertor board and mapping them to -1..+1
// -Neural preprocessing networks
// -Modular neural locomotion control
// -Motor command =  linear mapping from output neurons to motor positions

// Note that the parameter names are comparable to the ones described in the paper

// The controller is described in the paper
// Manoonpong, P.; Pasemann, F.; Woergoetter, F. (2007) 
// Sensor-Driven Neural Control for Omnidirectional Locomotion 
// and Versatile Reactive Behaviors of Walking Machines. Robotics 
// and Autonomous Systems, Elsevier Science, in press.


// Created by P.Manoonpong
////////////////////////////////////////////




/**
 * @author Poramate 1.02.2007
 */

#include "Sensor-DrivenNeuralController.h"
#include "time.h"


// ----------------------------------------------------------------------
// ------------ constructor ---------------------------------------------
// ----------------------------------------------------------------------

Serial::Serial()
{


// ------------Initial Serail -------------------------------------------

 hComp = CreateFile( COMPORT,
    GENERIC_READ | GENERIC_WRITE,     //Reading | Writing file
    0, 
    NULL,                             //Not protect attribute
    OPEN_EXISTING,                    //Open communication
    0,                                //No verlapped I/O
    NULL                              //Use NULL to communication
    );

 if (hComp == INVALID_HANDLE_VALUE) 
 {
   printf("File error\n");
 }

 valReturn = GetCommState(hComp, &dcb);

 if (!valReturn) 
 {
    printf("Com port state error\n");
 }

 dcb.BaudRate = CBR_57600;//115200; //baudrate 
 dcb.ByteSize = 8;         //8 data Bits
 dcb.Parity = NOPARITY;    //No parity(parity is used to detect the transmission error)
 dcb.StopBits = ONESTOPBIT; //1 Stop bit

 valReturn = SetCommState(hComp, &dcb);
  //Define value for communication

 if (!valReturn) {
   printf("Com port error\n");
 }

saveFile1.open("ReadSensors.txt",ios::out);
//////////////////////////////////End Serial initial/////////////////////////////////////////





//////////////////////////////////Initial parameters///////////////////////////////////////////


//****Initial First Module of Neural Preprocessor Sigmoid (NP1)****///

	activityP1 = 0.0;
    outputP1 = 0.0;

//****Initial second Module of Neural Preprocessor Sigmoid (NP2)****///

	activityP9 = 0.0;
	outputP9 = 0.0;


//****Initial Third Module of Neural Preprocessor Sigmoid (NP3)****///

	activityP2 = 0.0;
	activityP3 = 0.0;
	outputP2 = 0.0;
	outputP3 = 0.0;

//--Initial XNOR Sigmoid(Subnetwork of Second module) --///

	
	activityP4 = 0.0;
	activityP5 = 0.0;
	activityP6 = 0.0;
	outputP4 = 0.0; 
	outputP5 = 0.0; 
	outputP6 = 0.0;
	

//****Initial Forth Module of Neural Preprocessor Tanh (NP4)****///

	activityP7 = -1.0;  
	activityP8 = -1.0; 
	outputP7   = -1.0; 
	outputP8   = -1.0; 

	activity_HiddenL = -1.0;  
    activity_HiddenR = -1.0;
	output_HiddenL = -1.0;
	output_HiddenR = -1.0;





//****Initial CPG network of Neural Locomotion Control Tanh****///

	activityH1 = 0.0;
	activityH2 = 0.0;

	outputH1 = 0.0;
	outputH2 = 0.0;


//****Initial PSN of Neural Locomotion Control Tanh****///

	activityH3 = 0.0;
	activityH4= 0.0;

	activityH5= 0.0;
	activityH6= 0.0;
	activityH7= 0.0;
	activityH8= 0.0;

	activityH9= 0.0;
	activityH10= 0.0;
	activityH11= 0.0;
	activityH12= 0.0;

	activityH13= 0.0;
	activityH14= 0.0;

	outputH3= 0.0;
	outputH4= 0.0;
	outputH5= 0.0;
	outputH6= 0.0;
	outputH7= 0.0;
	outputH8= 0.0;
	outputH9= 0.0;
	outputH10= 0.0;
	outputH11= 0.0;
	outputH12= 0.0;

	outputH13= 0.0;
	outputH14= 0.0;


/****Initial VRNs of Neural Locomotion Control Tanh****///

 //-----------VRN1_L-------------------------------//
	activityH15 = 0.0;
	activityH16 = 0.0; //*****Input4

	activityH19 = 0.0;
	activityH20 = 0.0;
	activityH21 = 0.0;
	activityH22 = 0.0;

	activityH27 = 0.0;

	outputH15 = 0.0;
	outputH16 = 0.0;

	outputH19 = 0.0;
	outputH20 = 0.0;
	outputH21 = 0.0;
	outputH22 = 0.0;

	outputH27 = 0.0;


 //-----------VRN2_R-------------------------------//

	activityH17 = 0.0 ;
	activityH18 = 0.0; //*****Input5



	activityH23 = 0.0;
	activityH24 = 0.0;
	activityH25 = 0.0;
	activityH26 = 0.0;

	activityH28 = 0.0;


	outputH17 = 0.0;
	outputH18 = 0.0;


	outputH23 = 0.0;

	outputH24 = 0.0;

	outputH25 = 0.0;
	outputH26 = 0.0;
	outputH28 = 0.0;


/****Initial Motor neurons Tanh****///

	
	activityFL1 = 0.0;
	activityFL2 = 0.0;
	activityFL3 = 0.0;

	activityFR1 = 0.0;
	activityFR2 = 0.0;
	activityFR3 = 0.0;

	activityCL1 = 0.0;
	activityCL2 = 0.0;
	activityCL3 = 0.0;

	activityCR1 = 0.0;
	activityCR2 = 0.0;
	activityCR3 = 0.0;


	activityTL1 = 0.0;
	activityTL2 = 0.0;
	activityTL3 = 0.0;

	activityTR1 = 0.0;
	activityTR2 = 0.0;
	activityTR3 = 0.0;




	//MID motors
	activityBJ = 0.0;

	//lower tail 
	activityLT = 0.0;

	//upper tail
	activityUT = 0.0;


	outputFL1 = 0.0;
	outputFL2 = 0.0;
	outputFL3 = 0.0;

	outputFR1 = 0.0;
	outputFR2 = 0.0;
	outputFR3 = 0.0;

	outputCL1 = 0.0;
	outputCL2 = 0.0;
	outputCL3 = 0.0;

	outputCR1 = 0.0;
	outputCR2 = 0.0;
	outputCR3 = 0.0;

	outputTL1 = 0.0;
	outputTL2 = 0.0;
	outputTL3 = 0.0;

	outputTR1 = 0.0;
	outputTR2 = 0.0;
	outputTR3 = 0.0;

	outputBJ = 0.0;
	outputLT = 0.0;
	outputUT = 0.0;


//***********************************************//
	

	Input = 0.;
    counter = 0;
	IRMR_outputLowpass = 0.;
    IRML_outputLowpass = 0.;
	IRRP_outputLowpass = 0.;

//////////////////////////////////End initial variables/////////////////////////////////////////



}

// ----------------------------------------------------------------------
// ------------ destructor ----------------------------------------------
// ----------------------------------------------------------------------

Serial::~Serial()
{

}



////////////////////////JOYSTICK CONTROL/////////////////////////////////////////////
// Added by NateW
//
// Read the joystick X, Y, Z, and R positions, and scale them to +/- 1.0.
// Results are stored in the application object's m_ControlInput member.

#pragma warning(disable: 4201) // nonstandard extension used : nameless struct/union
#include "mmsystem.h" 
#pragma warning(default: 4201)
typedef double real;

class ControlInput
{
public:
	real m_rJoystickX;
	real m_rJoystickY;
	real m_rJoystickZ;
	real m_rJoystickR;
};

static ControlInput g_ControlInput;

void Serial::RefreshControlInput ()
{
	JOYINFOEX state;
	MMRESULT mmr = 0;

	state.dwSize = sizeof (JOYINFOEX);
	state.dwFlags = JOY_RETURNALL;

	mmr = joyGetPosEx (JOYSTICKID1, &state);

    
////////////Read joystick information////////////////////////
JoystickX = (state.dwXpos/30000);
JoystickY = (state.dwYpos/30000);
JoystickZ = (state.dwZpos/30000);
PressButton = state.dwButtons;
////////////////////////////////////////////////////////////


/**********Not important****************/
#define GetData(x) \
	g_ControlInput.m_rJoystick##x = state.dw##x##pos; \
	g_ControlInput.m_rJoystick##x /= (0xFFFF / 2); \
	g_ControlInput.m_rJoystick##x -= 1;

	 
	 // Si added a bit of error handling here for those of us without joysticks.
	 // (the UI goes a bit nuts otherwise)
	 
	if(mmr == JOYERR_NOERROR)
	{
		GetData(X);
		GetData(Y);
		GetData(Z);
		GetData(R);
	}
	else
		memset(&g_ControlInput, 0, sizeof(ControlInput));

/**********Not important****************/

}

// End of Joystick control
////////////////////////////////////////////////////////////////////////////





// ----------------------------------------------------------------------
// ------------ GetSensors ----------------------------------------------
// ----------------------------------------------------------------------
void Serial::GetSensors()
{

/////define parameters///////////////////////
double potValue[34];
int success;
char serial_msg[100];
int index=0;
double sensor1=1;
/////End define parameters///////////////////


/////define detection range of sensor signals (Max, Min values)//////////


double AvergRightIR_Min = 25.0;
double AvergRightIR_Max = 57.0;


double AvergLeftIR_Min = 25.0;
double AvergLeftIR_Max = 57.0;

//Infra-Red sensors/////////////////////////////////////////////////////

//Right sensor
int	 IRmaxR = 130;//AR Antenna Right
int	 IRminR = 95;


int	 IR_FRmaxR = 60; //**  Front Right    s[1] poti26
int	 IR_FRminR = 45;

int	 IR_MRmaxR = 100;//** Mid Right s[2] poti28
int	 IR_MRminR = 50;

int	 IR_BRmaxR = 100;
int	 IR_BRminR = 70; 


// Left sensor
int	 IRmaxL = 130;  //AL Antenna Left   s[3] poti 25
int	 IRminL = 95;

int	 IR_FLmaxL = 60;//**   Front Left     s[4] poti 27
int	 IR_FLminL = 45;//

int	 IR_MLmaxL = 100;//**  Mid Left   s[5] poti 24
int	 IR_MLminL = 50;

int	 IR_BLmaxL = 100;
int	 IR_BLminL = 50;
////////////////////////////////////////////////////////////////////////


int LowpassGain = 45;


// Position sensor////UD sensor/////////////////////////////////////////
int	 Switchmax = 200; // s[6] poti 22
int	 Switchmin = 5;
///////////////////////////////////////////////////////////////////////


//Foot contact sensors/////////////////////////////////////////////////////
int	 Footmax = 90;//5; // s[6] poti 22
int	 Footmin = 160;//200;
double Amplifiy_factor_Foot= 10.;

///////////////////////////////////////////////////////////////////////


double Amplifiy_factor = 2.5;

/////End define detection range of sensor signals (Max, Min values)///////



//**********************************************************************//
//		Sensory reading from MBOARD										//
//**********************************************************************//
// ################## read potentiometer ###################
//PurgeComm(hComp,(PURGE_TXCLEAR | PURGE_RXCLEAR));


 do
{


		char comByte=2;
		  char end=0;
		  //Sending "getSensors" command to the board
		  sprintf(serial_msg, "%c%c",comByte,end);
		  WriteFile(hComp,&serial_msg,2 ,&nWritePort, NULL); 

  
		  // --- Reading the potentiometer values
		  for (int i=1;i<34;i++)
		  {
			 do
			 {
			   success=ReadFile(hComp,&chBuff,1,&nReadPort,NULL);
			   if (success)
			   {
				 potValue[i]=(unsigned char)(chBuff); 
			   }  
			 //  printf("debug -> %d   %d\n",i,potValue[i]);
			 }
			 while(!success);
		  } 
  
		 }while((unsigned char)(chBuff)!=0);// "0" is sync byte




//**********************************************************************//
// Convert sensory signal to -1 and 1  with linear mapping, Right, Left	//
//**********************************************************************//


 //**********Right detection*******************//

 //IR sensor Front Right (Antenna)


  Sensor[0]=potValue[29];// puppetplayer (28)

  if (Sensor [0]<0)
  {
   Sensor [0]=Sensor [0]+256;
  }

  else

  Sensor[0]= Sensor[0];
 
 
   Sensor[1]=potValue[26]; // puppetplayer (25)

	
  if (Sensor [1]<0)
  {
   Sensor [1]=Sensor [1]+256;
  }

  else

   Sensor[1]=Sensor [1]; 
 
  
   AvergRightIR  = (Sensor[0]+Sensor[1])/2;
   
   AvergRightIR=(((AvergRightIR-AvergRightIR_Min)/(AvergRightIR_Max-AvergRightIR_Min))*2.0-1.0);
 
   if (AvergRightIR>1.0)
  {
   AvergRightIR=1;  
  }
  if (AvergRightIR<-1.0)
  {
   AvergRightIR=-1;
  }


 //**********Left dection*******************//

 
  //IR sensor Front Left (Antenna)

  Sensor[3]=potValue[25]; // 25 At pin Number 31 , it connects to IR sensor (24) pupetplayer
	
  if (Sensor [3]<0)
  {
   Sensor [3]=Sensor [3]+256;
  }

  else

   Sensor[3]= Sensor[3];


   Sensor[4]=potValue[27]; // puppetplayer (26)

  if (Sensor [4]<0)
  {
   Sensor [4]=Sensor[4]+256;
  }

  else

  Sensor[4]= Sensor[4];


   AvergLeftIR  = (Sensor[3]+Sensor[4])/2;

   AvergLeftIR = (((AvergLeftIR- AvergLeftIR_Min)/(AvergLeftIR_Max- AvergLeftIR_Min))*2.0-1.0);
 
   if ( AvergLeftIR>1.0)
  {
    AvergLeftIR=1;  
  }
  if ( AvergLeftIR<-1.0)
  {
    AvergLeftIR=-1;
  }



  //**************************************************************************//
  // Convert sensory signal to 0 and 1  with linear mapping, Right, Left, Rear//
  //**************************************************************************//

  //**********Side Right dection*******************//

  //IR sensor Middle Right Leg 

  Sensor[2]=potValue[28];// puppetplayer (27)

  if (Sensor [2]<0)
  {
   Sensor [2]=Sensor[2]+256;
  }

  else

  Sensor[2]=(((Sensor[2]-IR_MRminR)/(IR_MRmaxR -IR_MRminR)));
 
	
  if (Sensor[2]>1)
  {
   Sensor[2]=1;
  }
  if (Sensor[2]<0)
  {
   Sensor[2]=0;
  }



  //***Low-pass filter of IR signal*********//


	IRMR_outputLowpass = (Sensor[2]+LowpassGain*IRMR_outputLowpass)/(LowpassGain+1);


  //**********Side Left dection*******************//

  //IR sensor Middle Left Leg 

  Sensor[5]=potValue[24];  //puppetplayer (23)

  if (Sensor [5]<0)
  {
   Sensor [5]=Sensor [5]+256;
  }

  else

   Sensor[5]=(((Sensor[5]-IR_MLminL)/(IR_MLmaxL-IR_MLminL))); 
 
   if (Sensor[5]>1)
  {
   Sensor[5]=1;
  }
  if (Sensor[5]<0)
  {
   Sensor[5]=0;
  }
   
	//***Low-pass filter of IR signal*********//


	IRML_outputLowpass = (Sensor[5]+LowpassGain*IRML_outputLowpass)/(LowpassGain+1);


//************ Rear sensor for speed control*******//

  Sensor[8]=potValue[19];  //puppetplayer (18)

  if (Sensor[8]<0)
  {
   Sensor[8]= Sensor[8]+256;
  }

  else

    Sensor[8]=(((Sensor[8]-IR_BLminL)/(IR_BLmaxL-IR_BLminL)));
 
   if (Sensor[8]>1)
  {
   Sensor[8]=1;
  }
  if (Sensor[8]<0)
  {
   Sensor[8]=0;
  }
   
  //***Low-pass filter of IR signal*********//


	IRRP_outputLowpass = (Sensor[8]+LowpassGain*IRRP_outputLowpass)/(LowpassGain+1);



  //**************************************************************************//
  // Convert sensory signal to -1 and 1 with linear mapping additional sensors//
  //**************************************************************************//

  
  //************ Upside-down position sensor for a Reflex behavior 0,..,1******//

  Sensor[6]=potValue[22]; 
  	
  if (Sensor [6]<0)
  {
   Sensor [6]=Sensor [6]+256;
  }

  else

  Sensor[6]=(((Sensor[6]-Switchmin)/(Switchmax-Switchmin)))*Amplifiy_factor; 
 
   if (Sensor[6]>1)
  {
   Sensor[6]=1;
  }
  if (Sensor[6]<0)
  {
   Sensor[6]=0;
  }


  //************ Foot R1 sensor -1..1***************************************//

  Sensor[10]= potValue[5];// puppetplayer (4)
  
  if (Sensor [10]<0)
  {
   Sensor [10]=Sensor [10]+256;
  }

  else

   Sensor[10]=(((Sensor[10]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
   if (Sensor[10]>1)
  {
   Sensor[10]=1;
  }
  if (Sensor[10]<-1)
  {
   Sensor[10]=-1;
  }

    //************ Foot R2 sensor -1..1***************************************//

  Sensor[11]= potValue[7];// puppetplayer (6) 
  
  if (Sensor [11]<0)
  {
   Sensor [11]=Sensor [11]+256;
  }

  else

    Sensor[11]=(((Sensor[11]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
   if (Sensor[11]>1)
  {
   Sensor[11]=1;
  }
  if (Sensor[11]<-1)
  {
   Sensor[11]=-1;
  }

    //************ Foot R3 sensor -1..1***************************************//

  Sensor[12]= potValue[9];// puppetplayer (8) 
  
  if (Sensor [12]<0)
  {
   Sensor [12]=Sensor [12]+256;
  }

  else

   Sensor[12]=(((Sensor[12]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
   if (Sensor[12]>1)
  {
   Sensor[12]=1;
  }
  if (Sensor[12]<-1)
  {
   Sensor[12]=-1;
  }
  

    //************ Foot L1 sensor -1..1***************************************//

  Sensor[13]= potValue[4]; // puppetplayer (3) 
  
  if (Sensor[13]<0)
  {
   Sensor[13]=Sensor [13]+256;
  }

  else

   Sensor[13]=(((Sensor[13]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
 
   if (Sensor[13]>1)
  {
   Sensor[13]=1;
  }
  if (Sensor[13]<-1)
  {
   Sensor[13]=-1;
  }
  
  

    //************ Foot L2 sensor -1..1***************************************//

  Sensor[14]= potValue[6]; // puppetplayer (5) 
  
  if (Sensor[14]<0)
  {
   Sensor[14]=Sensor[14]+256;
  }

  else

   Sensor[14]=(((Sensor[14]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
   if (Sensor[14]>1)
  {
   Sensor[14]=1;
  }
  if (Sensor[14]<-1)
  {
   Sensor[14]=-1;
  }
  

      //************ Foot L3 sensor -1..1***************************************//

  Sensor[15]= potValue[8]; // puppetplayer (7) 
  
  if (Sensor[15]<0)
  {
   Sensor[15]=Sensor[15]+256;
  }

  else

   Sensor[15]=(((Sensor[15]-Footmin)/(Footmax-Footmin))*2.0-1.0)*Amplifiy_factor_Foot; 
 
   if (Sensor[15]>1)
  {
   Sensor[15]=1;
  }
  if (Sensor[15]<-1)
  {
   Sensor[15]=-1;
  }
  
 

//**********************************************************************//
//		End Convert sensory signal to -1 and 1  mapping			     	//
//**********************************************************************//



}


// ----------------------------------------------------------------------
// ------------ Get Signal from joystick---------------------------------
// ----------------------------------------------------------------------

void Serial::GetJoystickSignal()

{


//**********************************************************************//
//		Joystick signals												//
//**********************************************************************//
   
///Not important///
	//	RefreshControlInput(); //JoyStick** 

	//	JoystickX = g_ControlInput.m_rJoystickX; //-1.0 Left, 1.0 Right, Marching (-0.007813)
	//	JoystickY = g_ControlInput.m_rJoystickY; //-1.0 Forwards ,  1.0 backward, Marching (-0.007813)
	//	JoystickZ = g_ControlInput.m_rJoystickZ;



	//Marching JoystickX = -0.007813, JoystickY = -0.007813; (press nothing)
	//Forwards JoystickX = -0.007813, JoystickY = -1.0;
	//Backward JoystickX = -0.007813, JoystickY = 1.0;
	//Left     JoystickX = -1.0, JoystickY = -0.007813;
	//Right    JoystickX = 1.0, JoystickY = -0.007813;
	//DiFRight JoystickX = 1.0, JoystickY = -1.0; 
	//DiBRight JoystickX = 1.0, JoystickY = 1.0; 
	//DiFLeft  JoystickX = -1.0, JoystickY = -1.0; 
	//DiBLeft  JoystickX = -1.0, JoystickY = 1.0; 
	//Press middle button JoystickX = 0.0, JoystickY = 0.0; 
///Not important////

///Actual Joystick Data///
	
	//Marching JoystickX = 1.0, JoystickY = 1.0; (press nothing)
	//Forwards JoystickX = 1.0, JoystickY = 0.0;
	//Backward JoystickX = 1.0, JoystickY = 2.0;
	//LateralLeft     JoystickX = 0.0, JoystickY = 1.0;
	//LateralRight    JoystickX = 2.0, JoystickY = 1.0;
	//DiFRight JoystickX = 2.0, JoystickY = 0.0; 
	//DiBRight JoystickX = 2.0, JoystickY = 2.0; 
	//DiFLeft  JoystickX = 0.0, JoystickY = 0.0; 
	//DiBLeft  JoystickX = 0.0, JoystickY = 2.0; 


	//PressButton = 1; button1
	//PressButton = 2; button2
	//PressButton = 4; button3
	//PressButton = 8; button4
	//PressButton = 16; button5
	//PressButton = 32; button6
	//PressButton = 64; button7
	//PressButton = 128; button8
	//PressButton = 256; buttonSE //Turn left
	//PressButton = 512; buttonST //Turn right


   //************Input to Locomotion Control (Joystick)*******//

     ///Actual Joystick Data///
	
    if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton ==0) //Marching
	{
    I1 = 0;   
    I2 = 1;  
	I3 = 0; 
    I4 = 0; //**
	I5 = 0; //**
	I6 = 0;
	
    }
	else if(JoystickX == 1.0 && JoystickY == 0.0 && PressButton ==0) //Forwards
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = -1; //**
	I5 = -1; //**
	I6 = 0;
    }
	else if(JoystickX == 1.0 && JoystickY == 2.0 && PressButton ==0) //Backwards
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = 1; //**
	I5 = 1; //**
	I6 = 0;
    }
    else if(JoystickX == 0.0 && JoystickY == 1.0 && PressButton ==0) //LateralLeft
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 1; 
    I4 = 0; //**
	I5 = 0; //**
	I6 = 0;
    }
    else if(JoystickX == 2.0 && JoystickY == 1.0 && PressButton ==0) //LateralRight
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 0; 
    I4 = 0; //**
	I5 = 0; //**
	I6 = 0;
    }
    else if(JoystickX == 2.0 && JoystickY == 0.0 && PressButton ==0) //DiFRight 
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 0; 
    I4 = -0.5; //**
	I5 = -0.5; //**
	I6 = 0;
    }
    else if(JoystickX == 2.0 && JoystickY == 2.0 && PressButton ==0) //DiBRight 
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 0; 
    I4 = 0.5; //**
	I5 = 0.5; //**
	I6 = 0;
    }
    else if(JoystickX == 0.0 && JoystickY == 0.0 && PressButton ==0) //DiFLeft
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 1; 
    I4 = -0.5; //**
	I5 = -0.5; //**
	I6 = 0;
    }
	else if(JoystickX == 0.0 && JoystickY == 2.0 && PressButton ==0) //DiBLeft
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 1; 
    I4 = 0.5; //**
	I5 = 0.5; //**
	I6 = 0;
    }
	
	
    else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 256) //Turn left buttonSE
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = 1; //**
	I5 = -1; //**
	I6 = 0;
    }
    
	else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 512) //Turn right buttonST
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = -1; //**
	I5 = 1; //**
	I6 = 0;
    }
    
	else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 16) //Forwards Left button5
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = -0.1; //**
	I5 = -1; //**
	I6 = 0;
    }
    else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 32) //TurnLeft with LateralLeft button6
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 1; 
    I4 = -0.5; //**
	I5 = 0.5; //**
	I6 = 0;
    }
 
    else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 64) //Forwards Right button7
	{
    I1 = 0;   
    I2 = 1;   
	I3 = 0; 
    I4 = -1; //**
	I5 = -0.1; //**
	I6 = 0;
    }
	else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 128) //TurnRight with LateralRight button8
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 0; 
    I4 = 0.5; //**
	I5 = -0.5; //**
	I6 = 0;
    }
    else if(JoystickX == 1.0 && JoystickY == 1.0 && PressButton == 8) //Speed up button4
	{
    I1 = 0;   
    I2 = 0;   
	I3 = 0; 
    I4 = -1; //**
	I5 = -1; //**
	I6 = 0.8;
    }


   


//printf("moving %f  %f %f\n", PressButton, JoystickX, JoystickY);//JoyStick**


}
// ----------------------------------------------------------------------
// ------------ Neural Preprocessor--------------------------------------
// ----------------------------------------------------------------------
void Serial::NeuralPreprocessor()

{

/*
Sensor[0] = IR1 (Front Right Antenna)*
Sensor[1] = IR2 (Front Right Leg)*
Sensor[2] = IR3 (Middle Right Leg)*

Sensor[3] = IR5 (Front Left Antenna)*
Sensor[4] = IR6 (Front Left Leg)*
Sensor[5] = IR7 (Middle Left Leg)*

Sensor[6] = Upside-down*
Sensor[8] = IR8 (Rear sensor for speed control)*
*/


//**********************************************************************//
//		Neural Preprocessing Networks									//
//**********************************************************************//
   
    //************Module 1 Preprocessing (Upside-down Reflex) Sigmoid 0..1 (NP1)*******//
 
	WeightP1_Upsidedown_sensor = 6.0;
	WeightP1_P1 = 7.2;
	BiaseP1 = -6.0;

	activityP1 = WeightP1_P1*outputP1+WeightP1_Upsidedown_sensor*Sensor[6]+BiaseP1;
	
	outputP1 = sigmoid(activityP1);// 

	
    //************Module 2 Preprocessing (Speed control rear IR sensor) sigmoid 0..1 (NP2) *******//
   
   WeightP9_IRSpeedControl = 6.0;
   WeightP9_P9 = 8.6;
   BiasP9      = -6.0;
   
   activityP9 = outputP9*WeightP9_P9+WeightP9_IRSpeedControl*Sensor[8]+BiasP9; 
   //activityP9 = outputP9*WeightP9_P9+WeightP9_IRSpeedControl*IRRP_outputLowpass+BiasP9; 
   outputP9   = sigmoid(activityP9);



  //************Module 3 Preprocessing (Hysteresis element of IR sensors at Middle legs) Sigmoid 0..1 (NP3)*******//
    


    WeightP2_IR = 6.0;   //Connection from IR to P8 
	WeightP3_IR = 6.0;   //Connection from IR to P9 
	WeightP2_P2  = 8.6;//10.0;	  //Self connection
	WeightP3_P3  = 8.6;//10.0;	  //Self connection
	BiasP2      = -6.0;//-7.0;	  //Bias or stationary Input
	BiasP3      = -6.0;//-7.0;	  //Bias or stationary Input



	activityP2 = WeightP2_P2*outputP2+WeightP2_IR*Sensor[5]+BiasP2;//(Middle Left Leg) 
	activityP3 = WeightP3_P3*outputP3+WeightP3_IR*Sensor[2]+BiasP3;//(Middle Right Leg)

	//activityP2 = WeightP2_P2*outputP2+WeightP2_IR*IRML_outputLowpass+BiasP2;//(Middle Left Leg) 
    //activityP3 = WeightP3_P3*outputP3+WeightP3_IR*IRMR_outputLowpass+BiasP3;//(Middle Right Leg)
   

	outputP2 = sigmoid(activityP2); // I_NP3_L2 Go to XNOR network 
    outputP3 = sigmoid(activityP3); // I_NP3_R2 Go to XNOR network and to I3


   //------- (XNOR net, training by BACK propragation, 0.7 learning rate for 2000 time steps) Sigmoid 0..1----//
  

	WeightP4_P2 = 6.29;    //Connection from P2 to P4
	WeightP5_P2 = -7.65;    //Connection from P2 to P5
    WeightP4_P3 = 6.29;    //Connection from P3 to P4
    WeightP5_P3 = -7.66;   //Connection from P3 to P5
	WeightP6_P4 = 15.45;   //Connection from P4 to P6 
    WeightP6_P5 = 15.26;   //Connection from P5 to P6 
   
    BiasP4      = -9.69;   //Bias or stationary Input 
	BiasP5      = 3.17;	   //Bias or stationary Input 
	BiasP6      = -7.57;   //Bias or stationary Input


	activityP4 = WeightP4_P2*outputP2+WeightP4_P3*outputP3+BiasP4;
	activityP5 = WeightP5_P2*outputP2+WeightP5_P3*outputP3+BiasP5;
	activityP6 = WeightP6_P4*outputP4+WeightP6_P5*outputP5+BiasP6;

	outputP4 = sigmoid(activityP4); //  XNOR network
    outputP5 = sigmoid(activityP5); //  XNOR network
	outputP6 = sigmoid(activityP6); //  XNOR Output ONP3 to I2 
   
	

    //************Module 4 Preprocessing (Average of IR inputs) Tanh -1..1 (NP4)*******//
 

   WeightP7_IR = 7.0;  //Connection from IR to P8
   WeightP8_IR = 7.0;  //Connection from IR to P9
   WeightP7_P7  = 5.4;	//Self connection
   WeightP8_P8  = 5.4;	//Self connection
   WeightP7_P8  = -3.55; //Connection from P9 to P8 
   WeightP8_P7  = -3.55; //Connection from P8 to P9 

   
   activityP7 = WeightP7_P7*outputP7+WeightP7_P8*outputP8+WeightP7_IR*AvergRightIR;//IR Right  
   activityP8 = WeightP8_P8*outputP8+WeightP8_P7*outputP7+WeightP8_IR*AvergLeftIR;// IR Left 


   outputP7 = tanh(activityP7); // Go to VRN to I4
   outputP8 = tanh(activityP8); // Go to VRN to I5
   


   //************Add more modules Here*******//
   
   
   
   
   
   //************Add more modules Here*******//

   
   //************Input to Modular Neural Locomotion Control (sensor-driven)*******//

   I1 =  outputP1;   // Reflex behavior (upside-down position) //Sigmoid 0..1
   I2 =  outputP6;   // Activating Fti joints //Sigmoid 0..1
   
   if(I2<0.3)
   {
    WalkingGain = 0.2;
   }
   else
    WalkingGain = 1.;
   
   I3 =  outputP3;   //outputP2;  // Phase switching (to left sideward) //Sigmoid 0,1


   I4 =  WalkingGain*outputP7;   // Right detection (turning left) I4 =+1 turn left, but IR on Right detection connect to I4 //Tanh -1..1
   I5 =  WalkingGain*outputP8;   // Left detection (turning right) I5 =+1 turn right, but IR on Left detection connect to I5 //Tanh -1..1
  

   I6 = outputP9;   // Speed control
    if(I6>0.5)
   {
    SpeedControlMax = 10;
   }
   else
    SpeedControlMax = 0;
   


}


// ----------------------------------------------------------------------
// ------------ Neural controller ---------------------------------------
// ----------------------------------------------------------------------


void Serial::ModularNeuralControl()

{

  //************Module 1 CPG neural oscillator*******//
    
  
	WeightH1_H1  = 1.5+I6*0.5;  
	WeightH2_H2  = 1.5+I6*0.5;
	WeightH1_H2  = 0.4+I6;
	WeightH2_H1  = -0.4-I6;
	

	BiasH1      = 0.01;
	BiasH2      = 0.01;

	activityH1 = WeightH1_H1*outputH1+WeightH1_H2*outputH2+BiasH1;
	activityH2 = WeightH2_H2*outputH2+WeightH2_H1*outputH1+BiasH2;

	outputH1 = tanh(activityH1);
	outputH2 = tanh(activityH2);

  //************Module 2 PSN Phase switching network*******//
	
    WeightH3_I3  = -1.0; 
	WeightH4_I3  = 1.0; 

   	WeightH5_H1  = 0.5; 
   	WeightH8_H1  = 0.5; 
	WeightH6_H2  = 0.5; 
   	WeightH7_H2  = 0.5;

	WeightH5_H3  = -5.0;
	WeightH7_H3  = -5.0;
	WeightH6_H4  = -5.0;
	WeightH8_H4  = -5.0;

	WeightH9_H5   = 0.5;
	WeightH10_H6  = 0.5;
	WeightH11_H7  = 0.5;
	WeightH12_H8  = 0.5;
	
	WeightH13_H9   = 3.0;
	WeightH13_H10  = 3.0;
	WeightH14_H11   = 3.0;
	WeightH14_H12  = 3.0;
	
	BiasH3       = 1.0;
	BiasH9       = 0.5;
	BiasH10      = 0.5;
	BiasH11      = 0.5;
	BiasH12      = 0.5;
	BiasH13      = -1.35;
	BiasH14      = -1.35;


	activityH3 = WeightH3_I3*I3+BiasH3; //*****Input3
	activityH4 = WeightH4_I3*I3;        //*****Input3

	activityH5 = WeightH5_H3*outputH3+WeightH5_H1*outputH1;
	activityH6 = WeightH6_H2*outputH2+WeightH6_H4*outputH4;
	activityH7 = WeightH7_H3*outputH3+WeightH7_H2*outputH2;
	activityH8 = WeightH8_H4*outputH4+WeightH8_H1*outputH1;

	activityH9 = WeightH9_H5*outputH5+BiasH9;
	activityH10 = WeightH10_H6*outputH6+BiasH10;
	activityH11 = WeightH11_H7*outputH7+BiasH11;
	activityH12 = WeightH12_H8*outputH8+BiasH12;

	activityH13 = WeightH13_H9*outputH9+WeightH13_H10*outputH10+BiasH13;
	activityH14 = WeightH14_H11*outputH11+WeightH14_H12*outputH12+BiasH14;

	outputH3 = tanh(activityH3);
	outputH4 = tanh(activityH4);
	outputH5 = tanh(activityH5);
	outputH6 = tanh(activityH6);
	outputH7 = tanh(activityH7);
	outputH8 = tanh(activityH8);
	outputH9 = tanh(activityH9);
	outputH10 = tanh(activityH10);
	outputH11 = tanh(activityH11);
	outputH12 = tanh(activityH12);
   	
	outputH13 = tanh(activityH13);
 	outputH14 = tanh(activityH14);


   //************Module 3 VRNs velocity regulating network*******//
 
	//-----------VRN1_L-------------------------------//

    WeightH15_H14  = 1.75; //VRN1_L (Y input)
	WeightH16_I4  = 5.0; //VRN1_L (X input)

    WeightH19_H15  = 1.7246;   //VRN1_L
    WeightH20_H15  = -1.7246;  //VRN1_L
    WeightH21_H15  = 1.7246;   //VRN1_L
	WeightH22_H15  = -1.7246;  //VRN1_L

	WeightH19_H16  = 1.7246;   //VRN1_L
	WeightH20_H16  =-1.7246;   //VRN1_L
	WeightH21_H16  =-1.7246;   //VRN1_L
   	WeightH22_H16  = 1.7246;   //VRN1_L

	WeightH27_H19  = 0.5;   //VRN1_L
	WeightH27_H20  = 0.5;   //VRN1_L
	WeightH27_H21  = -0.5;   //VRN1_L
	WeightH27_H22  = -0.5;   //VRN1_L

	BiasH19      = -2.48285;
    BiasH20      = -2.48285;
	BiasH21      = -2.48285;
	BiasH22      = -2.48285;	


	activityH15 = WeightH15_H14*outputH14;
	activityH16 = WeightH16_I4*I4; //*****Input4

	activityH19 = WeightH19_H15*outputH15+WeightH19_H16*outputH16+BiasH19;
    activityH20 = WeightH20_H15*outputH15+WeightH20_H16*outputH16+BiasH20;
	activityH21 = WeightH21_H15*outputH15+WeightH21_H16*outputH16+BiasH21;
	activityH22 = WeightH22_H15*outputH15+WeightH22_H16*outputH16+BiasH22;

    activityH27 = WeightH27_H19*outputH19+WeightH27_H20*outputH20+WeightH27_H21*outputH21+WeightH27_H22*outputH22;

    outputH15 = tanh(activityH15);
	outputH16 = tanh(activityH16);

	outputH19 = tanh(activityH19);
	outputH20 = tanh(activityH20);
	outputH21 = tanh(activityH21);
	outputH22 = tanh(activityH22);

	outputH27 = tanh(activityH27);

   //-----------VRN2_R-------------------------------//

	WeightH17_H14  = 1.75; //VRN2_R (Y input)
	WeightH18_I5  = 5.0; //VRN1_R (X input)

	WeightH23_H17  = 1.7246;   //VRN2_R
    WeightH24_H17  = -1.7246;  //VRN2_R
    WeightH25_H17  = 1.7246;   //VRN2_R
	WeightH26_H17  = -1.7246;  //VRN2_R

	WeightH23_H18  = 1.7246;   //VRN2_R
	WeightH24_H18  =-1.7246;   //VRN2_R
	WeightH25_H18  =-1.7246;   //VRN2_R
   	WeightH26_H18  = 1.7246;   //VRN2_R

	WeightH28_H23  = 0.5;   //VRN2_R
	WeightH28_H24  = 0.5;   //VRN2_R
	WeightH28_H25  = -0.5;   //VRN2_R
	WeightH28_H26  = -0.5;   //VRN2_R

	BiasH23      = -2.48285;
    BiasH24      = -2.48285;
	BiasH25      = -2.48285;
	BiasH26      = -2.48285;

    
	activityH17 = WeightH17_H14*outputH14;
	activityH18 = WeightH18_I5*I5; //*****Input5



	activityH23 = WeightH23_H17*outputH17+WeightH23_H18*outputH18+BiasH23;
    activityH24 = WeightH24_H17*outputH17+WeightH24_H18*outputH18+BiasH24;
	activityH25 = WeightH25_H17*outputH17+WeightH25_H18*outputH18+BiasH25;
	activityH26 = WeightH26_H17*outputH17+WeightH26_H18*outputH18+BiasH26;

    activityH28 = WeightH28_H23*outputH23+WeightH28_H24*outputH24+WeightH28_H25*outputH25+WeightH28_H26*outputH26;

    
	outputH17 = tanh(activityH17);
	outputH18 = tanh(activityH18);
	
	outputH23 = tanh(activityH23);
	outputH24 = tanh(activityH24);
	outputH25 = tanh(activityH25);
	outputH26 = tanh(activityH26);

	outputH28 = tanh(activityH28);


   
    //************Motor neurons*******//

	//----------FL,R joints-------//

   	WeightFL1_H13  = -2.2; 
	WeightFL2_H13  = 2.2; 
	WeightFL3_H13  = -2.2; 

	WeightFR1_H13  = -2.2; 
	WeightFR2_H13  = 2.2; 
	WeightFR3_H13  = -2.2; 
    
	WeightFL1_I1  = 10.0; 
	WeightFL2_I1  = 10.0; 
	WeightFL3_I1  = 10.0; 

	WeightFR1_I1  = 10.0; 
	WeightFR2_I1  = 10.0; 
	WeightFR3_I1  = 10.0; 

	WeightFL1_I2  = -2.0; 
	WeightFL2_I2  = -2.0; 
	WeightFL3_I2  = -2.0; 

	WeightFR1_I2  = -2.0; 
	WeightFR2_I2  = -2.0; 
	WeightFR3_I2  = -2.0; 

	BiasFL1 = -2.0;
	BiasFL2 = -2.0;
	BiasFL3 = -2.0;

	BiasFR1 = -2.0;
	BiasFR2 = -2.0;
	BiasFR3 = -2.0;

	//---------CL,R joints----------//

	WeightCL1_H14  = -2.2; 
	WeightCL2_H14  = 2.2; 
	WeightCL3_H14  = -2.2; 

	WeightCR1_H14  = 2.2; 
	WeightCR2_H14  = -2.2; 
	WeightCR3_H14  = 2.2; 

	WeightCL1_I1  = 10.0; 
	WeightCL2_I1  = 10.0; 
	WeightCL3_I1  = 10.0; 

	WeightCR1_I1  = 10.0; 
	WeightCR2_I1  = 10.0; 
	WeightCR3_I1  = 10.0;

	BiasCL1 = -1.0;
	BiasCL2 = -1.0;
	BiasCL3 = -1.0;

	BiasCR1 = -1.0;
	BiasCR2 = -1.0;
	BiasCR3 = -1.0;

	//---------TL,R joints----------//
  
	WeightTL1_H27  = 1.3;
	WeightTL2_H27  = -1.3;
	WeightTL3_H27  = 1.3;

	WeightTR1_H28  = -1.3;
	WeightTR2_H28  = 1.3;
	WeightTR3_H28  = -1.3;

	WeightTL1_I1  = 10.0; 
	WeightTL2_I1  = -10.0; 
	WeightTL3_I1  = -10.0; 

	WeightTR1_I1  = 10.0; 
	WeightTR2_I1  = -10.0; 
	WeightTR3_I1  = -10.0;

	
	WeightBJ_H14 = 0.0; //BJ
	BiasBJ = -0.1;

	WeightLT_H14 = -2.5; //Lower tail
	BiasLT = -2.0;

	WeightUT_H14 = -2.5; //Upper tail
	BiasUT = 0.8;

	
	WeightLT_I1 = 10;
	WeightUT_I1 = -10;

	activityFL1 = WeightFL1_H13*outputH13+WeightFL1_I1*I1+WeightFL1_I2*I2+BiasFL1;
    activityFL2 = WeightFL2_H13*outputH13+WeightFL2_I1*I1+WeightFL2_I2*I2+BiasFL2;
	activityFL3 = WeightFL3_H13*outputH13+WeightFL3_I1*I1+WeightFL3_I2*I2+BiasFL3;

    activityFR1 = WeightFR1_H13*outputH13+WeightFR1_I1*I1+WeightFR1_I2*I2+BiasFR1;
    activityFR2 = WeightFR2_H13*outputH13+WeightFR2_I1*I1+WeightFR2_I2*I2+BiasFR2;
	activityFR3 = WeightFR3_H13*outputH13+WeightFR3_I1*I1+WeightFR3_I2*I2+BiasFR3;

	activityCL1 = WeightCL1_H14*outputH14+WeightCL1_I1*I1+BiasCL1;
	activityCL2 = WeightCL2_H14*outputH14+WeightCL2_I1*I1+BiasCL2;
	activityCL3 = WeightCL3_H14*outputH14+WeightCL3_I1*I1+BiasCL3;

	activityCR1 = WeightCR1_H14*outputH14+WeightCR1_I1*I1+BiasCR1;
	activityCR2 = WeightCR2_H14*outputH14+WeightCR2_I1*I1+BiasCR2;
	activityCR3 = WeightCR3_H14*outputH14+WeightCR3_I1*I1+BiasCR3;


	activityTL1 = WeightTL1_H27*outputH27+WeightTL1_I1*I1;
	activityTL2 = WeightTL2_H27*outputH27+WeightTL2_I1*I1;
	activityTL3 = WeightTL3_H27*outputH27+WeightTL3_I1*I1;

	activityTR1 = WeightTR1_H28*outputH28+WeightTR1_I1*I1;
	activityTR2 = WeightTR2_H28*outputH28+WeightTR2_I1*I1;
	activityTR3 = WeightTR3_H28*outputH28+WeightTR3_I1*I1;


		
	
	//MID motors
	activityBJ = WeightBJ_H14*outputH14+BiasBJ;

	//lower tail 
	activityLT = WeightLT_H14*-outputH14+WeightLT_I1*I1+BiasLT;

	//upper tail
	activityUT = WeightUT_H14*outputH14+WeightUT_I1*I1+BiasUT; 




   /////////////////
   outputCR1old = outCR1;
   outputCR2old = outCR2;
   outputCR3old = outCR3;

   outputCL1old = outCL1;
   outputCL2old = outCL2;
   outputCL3old = outCL3 ;

   /////////////////

	outputFL1 = tanh(activityFL1);
	outputFL2 = tanh(activityFL2);
	outputFL3 = tanh(activityFL3);

	outputFR1 = tanh(activityFR1);
	outputFR2 = tanh(activityFR2);
	outputFR3 = tanh(activityFR3);

	outputCL1 = outCL1 = tanh(activityCL1);
	outputCL2 = outCL2 = tanh(activityCL2);
	outputCL3 = outCL3 = tanh(activityCL3);

	outputCR1 = outCR1 = tanh(activityCR1);
	outputCR2 = outCR2 = tanh(activityCR2);
	outputCR3 = outCR3 = tanh(activityCR3);

	outputTL1 = tanh(activityTL1);
	outputTL2 = tanh(activityTL2);
	outputTL3 = tanh(activityTL3);

	outputTR1 = tanh(activityTR1);
	outputTR2 = tanh(activityTR2);
	outputTR3 = tanh(activityTR3);

	outputBJ = tanh(activityBJ);
	outputLT = tanh(activityLT);
	outputUT = tanh(activityUT);


  //------Convert to Motor output----//
/*
			if(outputCR1<outputCR1old)
	{
		outputCR1=-1.;
	}
	
	if(outputCR2<outputCR2old)
	{
		outputCR2=-1.;
	}
	
	
	if(outputCR3<outputCR3old)
	{
		outputCR3=-1.;
	}
	
	
	if(outputCL1<outputCL1old)
	{
		outputCL1=-1.;
	}
	
	if(outputCL2<outputCL2old)
	{
		outputCL2=-1.;
	}
	
	if(outputCL3<outputCL3old)
	{
		outputCL3=-1.;
	}
*/

	motorCom[0] = outputTR1;
	motorCom[1] = outputTR2;
	motorCom[2] = outputTR3;

  	motorCom[3] = outputTL1;
	motorCom[4] = outputTL2;
	motorCom[5] = outputTL3;
    
	motorCom[6] = outputCR1;
	motorCom[7] = outputCR2;
	motorCom[8] = outputCR3;

	motorCom[9] = outputCL1;
	motorCom[10] = outputCL2;
	motorCom[11] = outputCL3 ;

	
	motorCom[12] = outputFR1;
	motorCom[13] = outputFR2;
	motorCom[14] = outputFR3;

	motorCom[15] = outputFL1;
	motorCom[16] = outputFL2;
	motorCom[17] = outputFL3;

	motorCom[18] = outputBJ;
	motorCom[19] = outputLT;
	motorCom[20] = outputUT;


/*
		if(motorCom[6]<-0.45)
		{
		motorCom[6] =-1;
		}
		if(motorCom[7]<-0.45)
		{
		motorCom[7] =-1;
		}
		if(motorCom[8]<-0.45)
		{
		motorCom[8] =-1;
		}
		if(motorCom[9]<-0.45)
		{
		motorCom[9] =-1;
		}
		if(motorCom[10]<-0.45)
		{
		motorCom[10] =-1;
		}
		if(motorCom[11]<-0.45)
		{
		motorCom[11] =-1;
		}

*/

       
	    if(motorCom[6]>0.06)
		{
		motorCom[6] =0.06;
		}
		if(motorCom[7]>0.06)
		{
		motorCom[7] =0.06;
		}
		if(motorCom[8]>0.06)
		{
		motorCom[8] =0.06;
		}
		if(motorCom[9]>0.06)
		{
		motorCom[9] =0.06;
		}
		if(motorCom[10]>0.06)
		{
		motorCom[10] =0.06;
		}
		if(motorCom[11]>0.06)
		{
		motorCom[11] =0.06;
		}

   



		if(motorCom[6]<-0.3)
		{
		motorCom[6] =-1;
		}
		if(motorCom[7]<-0.3)
		{
		motorCom[7] =-1;
		}
		if(motorCom[8]<-0.3)
		{
		motorCom[8] =-1;
		}
		if(motorCom[9]<-0.3)
		{
		motorCom[9] =-1;
		}
		if(motorCom[10]<-0.3)
		{
		motorCom[10] =-1;
		}
		if(motorCom[11]<-0.3)
		{
		motorCom[11] =-1;
		}

   

}

// ----------------------------------------------------------------------
// --------------- Motor Command  ---------------------------------------
// ----------------------------------------------------------------------
void Serial::MotorCommand()

{
	int BiasDown;
    int BiasDownMiddleMin;
	int BiasDownMiddleMax;

	char serial_msg[100];

	char end =0; // Null as Sync-byte
	char comByte=1;
	int serialPos[33];
	int t;

	//BiasDown = 15;
BiasDown = -10;
BiasDownMiddleMin = 45;
BiasDownMiddleMax = 35;

// -------------------- initializing the Motor range ------------------------
	


	 // --- UPPER motors

	 servoPosMin[0] = 80;
	 servoPosMax[0] = 180;
	 
	 servoPosMin[1] = 125+10; 
	 servoPosMax[1] = 245-45+30;
	 
	 servoPosMin[2] = 110-60;
	 servoPosMax[2] = 210-40; 

	 servoPosMin[3] = 198;
	 servoPosMax[3] = 98;

	 servoPosMin[4] = 200;
	 servoPosMax[4] = 100+35-30;
	 
	 servoPosMin[5] = 130+60;
	 servoPosMax[5] = 30+40;


	// --- MIDDLE motors

	 servoPosMin[6] = 25+BiasDownMiddleMin ;
	 servoPosMax[6] = 175+BiasDownMiddleMax;

	 servoPosMin[7] = 30+BiasDownMiddleMin-28-SpeedControlMax-20;
	 servoPosMax[7] = 175+BiasDownMiddleMax;
	 
	 servoPosMin[8] = 20+BiasDownMiddleMin+5-SpeedControlMax;
	 servoPosMax[8] = 166+BiasDownMiddleMax+SpeedControlMax+30;

 	 
	 servoPosMin[9] = 225-BiasDownMiddleMin ;
	 servoPosMax[9] = 95-BiasDownMiddleMax;
 	 
	 servoPosMin[10] = 253-BiasDownMiddleMin+15+SpeedControlMax+10;
	 servoPosMax[10] = 103-BiasDownMiddleMax;

 	 
	 servoPosMin[11] = 230-BiasDownMiddleMin+SpeedControlMax;
	 servoPosMax[11] = 50-BiasDownMiddleMax-SpeedControlMax;



     //--- LOWER motor
	 servoPosMin[12] = 35+BiasDown;
	 servoPosMax[12] = 240;
	 
     servoPosMin[13] = 20+BiasDown+5;
	 servoPosMax[13] = 215;
	 
	 
	 servoPosMin[14] = 40+BiasDown+BiasDown;
	 servoPosMax[14] = 240;
	 


	 servoPosMin[15] = 220-BiasDown;
	 servoPosMax[15] = 30;
  
  
	 servoPosMin[16] = 225-BiasDown+5;
	 servoPosMax[16] = 25;
  
  
	 servoPosMin[17] = 238-BiasDown;
	 servoPosMax[17] = 20;

	 servoPosMin[18] = 5; 
	 servoPosMax[18] = 60;

	 servoPosMin[19] = 240;
	 servoPosMax[19] = 10;

	 servoPosMin[20] = 240;
	 servoPosMax[20] = 10;
	
	 // ##################### move motors ################



	 /////////////////////////
 
	 for (t=0;t<33;t++){
		serialPos[t]=128;//Setting all motor to midddle position 
	 }
	//////////////////////////



		//motorCom[0..11] = value from network (-1,0,1);
        //serialPos [] = Puppetplayer+1;

		//UPPER RIGHT servopos1[..+1]
		serialPos[20] =(int) (double)(((motorCom[0]+1.0)/2.0)*(servoPosMax[0]-servoPosMin[0])+servoPosMin[0]) ;  
		serialPos[25] =(int) (double)(((motorCom[1]+1.0)/2.0)*(servoPosMax[1]-servoPosMin[1])+servoPosMin[1]) ; 
		serialPos[31] =(int) (double)(((motorCom[2]+1.0)/2.0)*(servoPosMax[2]-servoPosMin[2])+servoPosMin[2]) ; 


		//UPPER LEFT
		serialPos[10] =(int) (double)(((motorCom[3]+1.0)/2.0)*(servoPosMax[3]-servoPosMin[3])+servoPosMin[3]) ; 
		serialPos[14] =(int) (double)(((motorCom[4]+1.0)/2.0)*(servoPosMax[4]-servoPosMin[4])+servoPosMin[4]) ; 
		serialPos[3] =(int) (double)(((motorCom[5]+1.0)/2.0)*(servoPosMax[5]-servoPosMin[5])+servoPosMin[5]) ; 


		//MID RIGHT
		serialPos[17] =(int) (double)(((motorCom[6]+1.0)/2.0)*(servoPosMax[6]-servoPosMin[6])+servoPosMin[6]) ; 
		serialPos[26] =(int) (double)(((motorCom[7]+1.0)/2.0)*(servoPosMax[7]-servoPosMin[7])+servoPosMin[7]) ; 
		serialPos[32] =(int) (double)(((motorCom[8]+1.0)/2.0)*(servoPosMax[8]-servoPosMin[8])+servoPosMin[8]) ; 


		//MID LEFT
		serialPos[11] =(int) (double)(((motorCom[9]+1.0)/2.0)*(servoPosMax[9]-servoPosMin[9])+servoPosMin[9]) ; 
		serialPos[13] =(int) (double)(((motorCom[10]+1.0)/2.0)*(servoPosMax[10]-servoPosMin[10])+servoPosMin[10]) ; 
		serialPos[4] =(int) (double)(((motorCom[11]+1.0)/2.0)*(servoPosMax[11]-servoPosMin[11])+servoPosMin[11]) ; 

		//LOW RIGHT
		serialPos[19] =(int) (double)(((motorCom[12]+1.0)/2.0)*(servoPosMax[12]-servoPosMin[12])+servoPosMin[12]) ; 
		serialPos[27] =(int) (double)(((motorCom[13]+1.0)/2.0)*(servoPosMax[13]-servoPosMin[13])+servoPosMin[13]) ; 
		serialPos[30] =(int) (double)(((motorCom[14]+1.0)/2.0)*(servoPosMax[14]-servoPosMin[14])+servoPosMin[14]) ; 


		//LOW LEFT
		serialPos[12] =(int) (double)(((motorCom[15]+1.0)/2.0)*(servoPosMax[15]-servoPosMin[15])+servoPosMin[15]) ; 
		serialPos[15] =(int) (double)(((motorCom[16]+1.0)/2.0)*(servoPosMax[16]-servoPosMin[16])+servoPosMin[16]) ; 
		serialPos[5] =(int) (double)(((motorCom[17]+1.0)/2.0)*(servoPosMax[17]-servoPosMin[17])+servoPosMin[17]) ; 




		//MID
		serialPos[18] = (int)(double)(((motorCom[18]+1.0)/2.0)*(servoPosMax[18]-servoPosMin[18])+servoPosMin[18]) ; 

		//Tail lower
		serialPos[7] =(int) (double)(((motorCom[19]+1.0)/2.0)*(servoPosMax[19]-servoPosMin[19])+servoPosMin[19]) ; 
		//Tail upper
		serialPos[8] =(int) (double)(((motorCom[20]+1.0)/2.0)*(servoPosMax[20]-servoPosMin[20])+servoPosMin[20]) ; 



	  //Sending message to command
	  sprintf(serial_msg, "%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c"
			,comByte,serialPos[1],serialPos[2]
				  ,serialPos[3],serialPos[4]
				  ,serialPos[5],serialPos[6]
				  ,serialPos[7],serialPos[8]
				  ,serialPos[9],serialPos[10]
				  ,serialPos[11],serialPos[12]
				  ,serialPos[13],serialPos[14]
				  ,serialPos[15],serialPos[16]
				  ,serialPos[17],serialPos[18]
				  ,serialPos[19],serialPos[20]
				  ,serialPos[21],serialPos[22]
				  ,serialPos[23],serialPos[24]
				  ,serialPos[25],serialPos[26]
				  ,serialPos[27],serialPos[28]
				  ,serialPos[29],serialPos[30]
				  ,serialPos[31],serialPos[32],end);

	   //Sendding command to serial port
	   WriteFile(hComp,&serial_msg,34 ,&nWritePort, NULL); 

}


double Serial::sigmoid(double num)
{

return 1./(1.+exp(-num));


}


double Serial::tanh(double num)
{

return 2./(1.+exp(-2.*num))-1.0;

}








